=== modified file 'grackle/client.py'
--- grackle/client.py	2012-01-30 20:37:22 +0000
+++ grackle/client.py	2012-03-16 19:37:09 +0000
@@ -6,19 +6,11 @@
     urlencode,
 )
 
-
-class UnsupportedDisplayType(Exception):
-    """Raised when an Unsupported display_type is requested."""
-
-
-class UnsupportedOrder(Exception):
-    """Raised when an Unsupported order is requested."""
-
-
-SUPPORTED_DISPLAY_TYPES = (
-    'all',
-    'text-only',
-    'headers-only',
+from grackle.error import (
+    MessageIdNotFound,
+    UnparsableDateRange,
+    UnsupportedDisplayType,
+    UnsupportedOrder,
     )
 
 
@@ -55,6 +47,22 @@
         connection.request(method, url, body)
         return connection.getresponse()
 
+    def put_archive(self, archive_id, mbox=None):
+        """Create an archive.
+
+        :param archive_id: The archive id.
+        :param mbox: An optional mbox with messages to add to the new archive.
+        """
+        response = self._method_archive(
+            'POST', '', {'archive_id': archive_id}, None)
+        response.read()
+        if response.status == httplib.BAD_REQUEST:
+            raise Exception('wtf')
+        elif response.status == httplib.CREATED:
+            return
+        else:
+            raise Exception('!!')
+
     def put_message(self, archive_id, key, file_obj):
         """Put a message into an archive.
 
@@ -63,8 +71,9 @@
             the message.
         :param file_obj: The raw text of the message, as a file.
         """
+        path = '%s/%s' % (archive_id, key)
         response = self._method_archive(
-            'POST', archive_id, {'key': key}, file_obj.read())
+            'POST', path, {}, file_obj.read())
         response.read()
         if response.status == httplib.BAD_REQUEST:
             raise Exception('wtf')
@@ -73,14 +82,18 @@
         else:
             raise Exception('!!')
 
-    def get_messages(self, archive_id, message_ids=None, limit=None,
-                     memo=None, order=None, headers=None,
-                     max_body_length=None, include_hidden=False,
+    def get_messages(self, archive_id, message_ids=None, date_range=None,
+                     limit=None, memo=None, order=None, headers=None,
+                     include_hidden=False, max_body_length=None,
                      display_type='all'):
         """Retrieve specified messages.
 
         :param archive_id: The archive to retrieve messages from.
         :param message_ids: (optional) Retrieve only messages with these ids.
+        :param date_range: Retrieve the messages from or between a range of
+            dates. Example: 2012-01-01..2012-01-31 retrieve all the messages
+            between the 01 and 31 of January, including message from 01
+            and 31.
         :param limit: The maximum number of messages to return.  The server
             may, at its discretion, return fewer.
         :param memo: (optional) Opaque identifier describing the position in
@@ -108,6 +121,8 @@
         parameters = {}
         if message_ids is not None:
             parameters['message_ids'] = message_ids
+        if date_range is not None:
+            parameters['date_range'] = date_range
         if limit is not None:
             parameters['limit'] = limit
         if memo is not None:
@@ -118,13 +133,31 @@
             parameters['headers'] = headers
         if max_body_length is not None:
             parameters['max_body_length'] = max_body_length
-        if display_type not in SUPPORTED_DISPLAY_TYPES:
-            raise UnsupportedDisplayType
         parameters['display_type'] = display_type
         parameters['include_hidden'] = include_hidden
         query = {'parameters': simplejson.dumps(parameters)}
         response = self._method_archive('GET', archive_id, query)
         if response.status == httplib.BAD_REQUEST:
-            raise UnsupportedOrder
+            if response.reason == UnsupportedOrder.__doc__:
+                raise UnsupportedOrder
+            elif response.reason == UnsupportedDisplayType.__doc__:
+                raise UnsupportedDisplayType
+            elif response.reason == UnparsableDateRange.__doc__:
+                raise UnparsableDateRange
+            else:
+                raise ValueError('Bad request')
+        data = response.read()
+        return simplejson.loads(data)
+
+    def hide_message(self, archive_id, message_id, hidden):
+        parameters = {
+            'hidden': hidden,
+            }
+        query = {'parameters': simplejson.dumps(parameters)}
+        path = '%s/%s' % (archive_id, message_id)
+        response = self._method_archive('POST', path, query)
+        if response.status == httplib.BAD_REQUEST:
+            if response.reason == MessageIdNotFound.__doc__:
+                raise MessageIdNotFound
         data = response.read()
         return simplejson.loads(data)

=== added file 'grackle/error.py'
--- grackle/error.py	1970-01-01 00:00:00 +0000
+++ grackle/error.py	2012-03-01 01:08:15 +0000
@@ -0,0 +1,25 @@
+__all__ = [
+    'MessageIdNotFound',
+    'UnparsableDateRange',
+    'UnsupportedDisplayType',
+    'UnsupportedOrder',
+    ]
+
+
+class ArchiveIdNotFound(Exception):
+    """No archive matching the archive_id was found in the store."""
+
+class MessageIdNotFound(Exception):
+    """No message matching the message_id was found in the archive."""
+
+
+class UnparsableDateRange(Exception):
+    """The date_range was not in the format of 2012-01-01..2012-01-31."""
+
+
+class UnsupportedDisplayType(Exception):
+    """Raised when an Unsupported display_type is requested."""
+
+
+class UnsupportedOrder(Exception):
+    """Raised when an Unsupported order is requested."""

=== added file 'grackle/store.py'
--- grackle/store.py	1970-01-01 00:00:00 +0000
+++ grackle/store.py	2012-03-16 19:01:21 +0000
@@ -0,0 +1,210 @@
+__all__ = [
+    'MemoryStore',
+    ]
+
+import email
+import simplejson
+from urlparse import parse_qs
+
+from grackle.error import (
+    ArchiveIdNotFound,
+    MessageIdNotFound,
+    UnparsableDateRange,
+    UnsupportedDisplayType,
+    UnsupportedOrder,
+    )
+
+
+SUPPORTED_DISPLAY_TYPES = set(['all', 'text-only', 'headers-only'])
+
+
+SUPPORTED_ORDERS = set(
+    ['date', 'author', 'subject', 'thread_newest', 'thread_oldest',
+     'thread_subject'])
+
+
+def threaded_messages(messages):
+    threads = {}
+    count = 0
+    pending = []
+    for message in messages:
+        if message.get('replies') is None:
+            threads[message['message_id']] = [message]
+            count += 1
+        else:
+            pending.append(message)
+    for message in pending:
+        threads[message['replies']].append(message)
+    return threads.values()
+
+
+def get_body_text(message):
+    """Return the first plain/text messaage part."""
+    if not message.is_multipart():
+        return message.get_payload()
+    for part in email.iterators.typed_subpart_iterator(message, 'multipart'):
+        subparts = part.get_payload()
+        for subpart in subparts:
+            if subpart.get_content_type() == 'text/plain':
+                return subpart.get_payload().strip()
+    return ''
+
+
+def make_json_message(message_id, raw_message, hidden=False):
+    message = email.message_from_string(raw_message)
+    headers = dict(message.items())
+    message = {
+        'message_id': message_id,
+        'headers': headers,
+        # This is broken because the in-reply-to must be encoded.
+        # X-Message-ID-Hash is calculated from the Base 32.
+        'thread_id': headers.get('in-reply-to', message_id),
+        'date': headers.get('date'),
+        'subject': headers.get('subject'),
+        'author': headers.get('from'),
+        'hidden': hidden,
+        'attachments': [],
+        'replies': headers.get('in-reply-to'),
+        'body': get_body_text(message),
+        }
+    return message
+
+
+class MemoryStore:
+    """A memory-backed message store."""
+
+    def __init__(self, message_archives):
+        """Constructor."""
+        self.message_archives = message_archives
+
+    @staticmethod
+    def is_multipart(message):
+        return isinstance(message['body'], list)
+
+    def put_message(self, archive_id, message_id, raw_message):
+        # XXX sinzui 2012-02-29: this needs to raise an error
+        # if the th archive_id is invalid, message_id is not base32
+        # or the raw message is not an email.
+        if archive_id not in self.message_archives:
+            raise ArchiveIdNotFound()
+        if not raw_message:
+            raise ValueError('raw_message is not a message.')
+        json_message = make_json_message(message_id, raw_message)
+        messages = self.message_archives[archive_id]
+        messages.append(json_message)
+
+    def get_messages(self, archive_id, query_string):
+        """Return matching messages.
+
+        :param archive_id: The archive to retrieve from.
+        :param query_string: Contains 'parameters', which is a JSON-format
+            string describing parameters.
+        """
+        query = parse_qs(query_string)
+        parameters = simplejson.loads(query['parameters'][0])
+        order = parameters.get('order')
+        messages = self.message_archives[archive_id]
+        if order is not None:
+            if order not in SUPPORTED_ORDERS:
+                raise UnsupportedOrder
+            elif order.startswith('thread_'):
+                threaded = threaded_messages(messages)
+                messages = []
+                if order == 'thread_subject':
+                    threaded.sort(key=lambda t: t[0]['subject'])
+                if order == 'thread_oldest':
+                    threaded.sort(key=lambda t: min(m['date'] for m in t))
+                if order == 'thread_newest':
+                    threaded.sort(key=lambda t: max(m['date'] for m in t))
+                for thread in threaded:
+                    messages.extend(thread)
+            else:
+                messages.sort(key=lambda m: m[order])
+        display_type = parameters.get('display_type', 'all')
+        if display_type not in SUPPORTED_DISPLAY_TYPES:
+            raise UnsupportedDisplayType
+        if 'date_range' in parameters:
+            try:
+                start_date, end_date = parameters['date_range'].split('..')
+                if not start_date or not end_date:
+                    raise UnparsableDateRange
+            except ValueError:
+                raise UnparsableDateRange
+        new_messages = []
+        for message in messages:
+            if (not parameters['include_hidden'] and message['hidden']):
+                continue
+            if ('message_ids' in parameters
+                and message['message_id'] not in parameters['message_ids']):
+                continue
+            if ('date_range' in parameters
+                and (message['date'] < start_date
+                     or message['date'] > end_date)):
+                continue
+            message = dict(message)
+            if 'headers' in parameters:
+                headers = dict(
+                    (k, v) for k, v in message['headers'].iteritems()
+                    if k in parameters['headers'])
+                message['headers'] = headers
+            if display_type == 'headers-only':
+                del message['body']
+            elif display_type == 'text-only' and self.is_multipart(message):
+                text_parts = [
+                    part.get_payload() for part in message['body']
+                    if part.get_content_type() == 'text/plain']
+                message['body'] = '\n\n'.join(text_parts)
+            elif display_type == 'all' and self.is_multipart(message):
+                parts = [str(part.get_payload()) for part in message['body']]
+                message['body'] = '\n\n'.join(parts)
+            max_body = parameters.get('max_body_length')
+            if max_body is not None and display_type != 'headers-only':
+                message['body'] = message['body'][:max_body]
+            new_messages.append(message)
+        messages = new_messages
+        limit = parameters.get('limit', 100)
+        memo = parameters.get('memo')
+        message_id_indices = dict(
+            (m['message_id'], idx) for idx, m in enumerate(messages))
+        if memo is None:
+            start = 0
+        else:
+            start = message_id_indices[memo.encode('rot13')]
+        if start > 0:
+            previous_memo = messages[start - 1]['message_id'].encode('rot13')
+        else:
+            previous_memo = None
+        end = min(start + limit, len(messages))
+        if end < len(messages):
+            next_memo = messages[end]['message_id'].encode('rot13')
+        else:
+            next_memo = None
+        messages = messages[start:end]
+
+        response = {
+            'messages': messages,
+            'next_memo': next_memo,
+            'previous_memo': previous_memo
+            }
+        return response
+
+    def hide_message(self, archive_id, message_id, query_string):
+        """Change the visbility of a message in an archive.
+
+        :param archive_id: The archive to retrieve from.
+        :param query_string: Contains 'parameters', which is a JSON-format
+            string describing parameters.
+        """
+        query = parse_qs(query_string)
+        parameters = simplejson.loads(query['parameters'][0])
+        hidden = parameters['hidden']
+        messages = self.message_archives[archive_id]
+        for message in messages:
+            if message['message_id'] == message_id:
+                message['hidden'] = hidden
+            response = {
+                'message_id': message_id,
+                'hidden': hidden,
+                }
+            return response
+        raise MessageIdNotFound

=== modified file 'grackle/tests/test_client.py'
--- grackle/tests/test_client.py	2012-01-30 20:37:22 +0000
+++ grackle/tests/test_client.py	2012-03-16 19:01:21 +0000
@@ -2,6 +2,9 @@
     HTTPServer,
     BaseHTTPRequestHandler,
     )
+from email.message import Message
+from email.mime.multipart import MIMEMultipart
+from email.mime.text import MIMEText
 import httplib
 import logging
 import os
@@ -11,147 +14,83 @@
 import sys
 from unittest import TestCase
 from urlparse import urlparse
-from urlparse import parse_qs
 
 from testtools import ExpectedException
 
 from grackle.client import (
     GrackleClient,
+    UnparsableDateRange,
     UnsupportedDisplayType,
     UnsupportedOrder,
     )
-
-
-def threaded_messages(messages):
-    threads = {}
-    count = 0
-    pending = []
-    for message in messages:
-        if message.get('in_reply_to') is None:
-            threads[message['message_id']] = [message]
-            count += 1
-        else:
-            pending.append(message)
-    for message in pending:
-        threads[message['in_reply_to']].append(message)
-    return threads.values()
-
-
-class GrackleStore:
-    """A memory-backed message store."""
-
-    def __init__(self, messages):
-        """Constructor."""
-        self.messages = messages
-
-    def get_messages(self, archive_id, query_string):
-        """Return matching messages.
-
-        :param archive_id: The archive to retrieve from.
-        :param query_string: Contains 'parameters', which is a JSON-format
-            string describing parameters.
-        """
-        query = parse_qs(query_string)
-        parameters = simplejson.loads(query['parameters'][0])
-        order = parameters.get('order')
-        messages = self.messages[archive_id]
-        if order is not None:
-            if order not in SUPPORTED_ORDERS:
-                raise UnsupportedOrder
-            elif order.startswith('thread_'):
-                threaded = threaded_messages(messages)
-                messages = []
-                if order == 'thread_subject':
-                    threaded.sort(key=lambda t: t[0]['subject'])
-                if order == 'thread_oldest':
-                    threaded.sort(key=lambda t: min(m['date'] for m in t))
-                if order == 'thread_newest':
-                    threaded.sort(key=lambda t: max(m['date'] for m in t))
-                for thread in threaded:
-                    messages.extend(thread)
-            else:
-                messages.sort(key=lambda m: m[order])
-        display_type = parameters.get('display_type', 'all')
-        if display_type not in SUPPORTED_DISPLAY_TYPES:
-            raise UnsupportedDisplayType
-        new_messages = []
-        for message in messages:
-            if (not parameters['include_hidden']
-                and message.get('hidden', False)):
-                continue
-
-            if ('message_ids' in parameters
-                and message['message_id'] not in parameters['message_ids']):
-                continue
-            message = dict(message)
-            if 'headers' in parameters:
-                headers = dict(
-                    (k, v) for k, v in message['headers'].iteritems()
-                    if k in parameters['headers'])
-                message['headers'] = headers
-            max_body = parameters.get('max_body_length')
-            if max_body is not None:
-                message['body'] = message['body'][:max_body]
-            new_messages.append(message)
-        messages = new_messages
-        limit = parameters.get('limit', 100)
-        memo = parameters.get('memo')
-        message_id_indices = dict(
-            (m['message_id'], idx) for idx, m in enumerate(messages))
-        if memo is None:
-            start = 0
-        else:
-            start = message_id_indices[memo.encode('rot13')]
-        if start > 0:
-            previous_memo = messages[start - 1]['message_id'].encode('rot13')
-        else:
-            previous_memo = None
-        end = min(start + limit, len(messages))
-        if end < len(messages):
-            next_memo = messages[end]['message_id'].encode('rot13')
-        else:
-            next_memo = None
-        messages = messages[start:end]
-
-        response = {
-            'messages': messages,
-            'next_memo': next_memo,
-            'previous_memo': previous_memo
-            }
-        return response
+from grackle.store import (
+    make_json_message,
+    MemoryStore,
+    )
+
+
+def make_message(message_id, body='body', headers=None, hidden=False):
+    if headers is None:
+        headers = {}
+    message_headers = {
+        'Message-Id': message_id,
+        'date': '2005-01-01',
+        'subject': 'subject',
+        'from': 'author',
+        'replies': '',
+        }
+    message_headers.update(headers.items())
+    message = Message()
+    message.set_payload(body)
+    for key, value in message_headers.items():
+        message[key] = value
+    return make_json_message(message_id, message.as_string(), hidden)
+
+
+def make_mime_message(message_id, body='body', headers=None, hidden=False,
+                      attachment_type=None):
+    parts = MIMEMultipart()
+    parts.attach(MIMEText(body))
+    if attachment_type is not None:
+        attachment = Message()
+        attachment.set_payload('attactment data.')
+        attachment['Content-Type'] = attachment_type
+        attachment['Content-Disposition'] = 'attachment; filename="file.ext"'
+        parts.attach(attachment)
+    return make_message(message_id, parts.as_string(), headers, hidden)
 
 
 class ForkedFakeService:
     """A Grackle service fake, as a ContextManager."""
 
-    def __init__(self, port, messages=None, write_logs=False):
+    def __init__(self, port, message_archives=None, write_logs=False):
         """Constructor.
 
         :param port: The tcp port to use.
-        :param messages: A dict of lists of dicts representing messages.  The
-            outer dict represents the archive, the list represents the list of
-            messages for that archive.
+        :param message_archives: A dict of lists of dicts representing
+            archives of messages. The outer dict represents the archive,
+            the list represents the list of messages for that archive.
         :param write_logs: If true, log messages will be written to stdout.
         """
         self.pid = None
         self.port = port
-        if messages is None:
-            self.messages = {}
+        if message_archives is None:
+            self.message_archives = {}
         else:
-            self.messages = messages
+            self.message_archives = message_archives
         self.read_end, self.write_end = os.pipe()
         self.write_logs = write_logs
 
     @staticmethod
-    def from_client(client, messages=None):
+    def from_client(client, message_archives=None):
         """Instantiate a ForkedFakeService from the client.
 
         :param port: The client to provide service for.
-        :param messages: A dict of lists of dicts representing messages.  The
-            outer dict represents the archive, the list represents the list of
-            messages for that archive.
+        :param message_archives: A dict of lists of dicts representing
+            archives of messages. The outer dict represents the archive,
+            the list represents the list of messages for that archive.
         """
-        return ForkedFakeService(client.port, messages)
+        return ForkedFakeService(client.port, message_archives)
 
     def is_ready(self):
         """Tell the parent process that the server is ready for writes."""
@@ -172,8 +111,8 @@
     def start_server(self):
         """Start the HTTP server."""
         service = HTTPServer(('', self.port), FakeGrackleRequestHandler)
-        service.store = GrackleStore(self.messages)
-        for archive_id, messages in service.store.messages.iteritems():
+        service.store = MemoryStore(self.message_archives)
+        for archive_id, messages in service.store.message_archives.iteritems():
             for message in messages:
                 message.setdefault('headers', {})
         self.is_ready()
@@ -186,14 +125,6 @@
         os.kill(self.pid, SIGKILL)
 
 
-SUPPORTED_DISPLAY_TYPES = set(['all', 'text-only', 'headers-only'])
-
-
-SUPPORTED_ORDERS = set(
-    ['date', 'author', 'subject', 'thread_newest', 'thread_oldest',
-     'thread_subject'])
-
-
 class FakeGrackleRequestHandler(BaseHTTPRequestHandler):
     """A request handler that forwards to server.store."""
 
@@ -204,13 +135,36 @@
 
     def do_POST(self):
         """Create a message on POST."""
-        message = self.rfile.read(int(self.headers['content-length']))
-        if message == 'This is a message':
-            self.send_response(httplib.CREATED)
-            self.end_headers()
-            self.wfile.close()
+        scheme, netloc, path, params, query_string, fragments = (
+            urlparse(self.path))
+        parts = path.split('/')
+        if parts[1] != 'archive':
+            # This is an unknonwn operation?
+            return
+        if 'content-length' in self.headers:
+            operation = 'put_message'
         else:
-            self.send_error(httplib.BAD_REQUEST)
+            operation = 'hide_message'
+        if operation == 'put_message':
+            message = self.rfile.read(int(self.headers['content-length']))
+            try:
+                # This expected path is /archive/archive_id/message_id.
+                self.server.store.put_message(parts[2], parts[3], message)
+                self.send_response(httplib.CREATED)
+                self.end_headers()
+                self.wfile.close()
+            except:
+                self.send_error(httplib.BAD_REQUEST)
+        elif operation == 'hide_message':
+            try:
+                # This expected path is /archive/archive_id/message_id.
+                response = self.server.store.hide_message(
+                    parts[2], parts[3], query_string)
+                self.send_response(httplib.OK)
+                self.end_headers()
+                self.wfile.write(simplejson.dumps(response))
+            except:
+                self.send_error(httplib.BAD_REQUEST)
 
     def do_GET(self):
         """Retrieve a list of messages on GET."""
@@ -224,9 +178,9 @@
                 self.send_response(httplib.OK)
                 self.end_headers()
                 self.wfile.write(simplejson.dumps(response))
-            except UnsupportedOrder:
-                self.send_response(httplib.BAD_REQUEST)
-                self.wfile.write('Unsupported order')
+            except Exception, error:
+                self.send_response(
+                    httplib.BAD_REQUEST, error.__doc__)
                 return
 
     def log_message(self, format, *args):
@@ -239,12 +193,21 @@
 class TestPutMessage(TestCase):
 
     def test_put_message(self):
-        client = GrackleClient('localhost', 8436)
-        with ForkedFakeService.from_client(client):
-            client.put_message('arch1', 'asdf', StringIO('This is a message'))
+        client = GrackleClient('localhost', 8420)
+        message_archives = {'arch1': []}
+        with ForkedFakeService.from_client(client, message_archives):
+            client.put_message('arch1', 'id1', StringIO('This is a message'))
+            response = client.get_messages('arch1')
+        self.assertEqual(1, len(response['messages']))
+        message = response['messages'][0]
+        self.assertEqual('id1', message['message_id'])
+
+    def test_put_message_without_archive(self):
+        client = GrackleClient('localhost', 8421)
+        message_archives = {'arch1': []}
+        with ForkedFakeService.from_client(client, message_archives):
             with ExpectedException(Exception, 'wtf'):
-                client.put_message('arch1', 'asdf',
-                    StringIO('This is not a message'))
+                client.put_message('no-archive', 'id1', StringIO('message'))
 
 
 class TestGetMessages(TestCase):
@@ -257,11 +220,10 @@
             sorted(ids), sorted(messages, key=lambda m: m['message_id']))
 
     def test_get_messages(self):
-        client = GrackleClient('localhost', 8435)
-        with ForkedFakeService.from_client(client,
-            {'baz':
-            [{'message_id': 'foo'},
-             {'message_id': 'bar'}]}):
+        client = GrackleClient('localhost', 8430)
+        archive = {
+            'baz': [make_message('foo'), make_message('bar')]}
+        with ForkedFakeService.from_client(client, archive):
             response = client.get_messages('baz')
         self.assertEqual(['bar', 'foo'], sorted(m['message_id'] for m in
             response['messages']))
@@ -270,20 +232,17 @@
 
     def test_get_messages_by_id(self):
         client = GrackleClient('localhost', 8437)
-        with ForkedFakeService.from_client(client,
-            {'baz':
-            [{'message_id': 'foo'},
-             {'message_id': 'bar'}]}):
+        archive = {
+            'baz': [make_message('foo'), make_message('bar')]}
+        with ForkedFakeService.from_client(client, archive):
             response = client.get_messages('baz', message_ids=['foo'])
         message, = response['messages']
         self.assertEqual('foo', message['message_id'])
 
     def test_get_messages_batching(self):
         client = GrackleClient('localhost', 8438)
-        with ForkedFakeService.from_client(client,
-            {'baz':
-            [{'message_id': 'foo'},
-             {'message_id': 'bar'}]}):
+        archive = {'baz': [make_message('foo'), make_message('bar')]}
+        with ForkedFakeService.from_client(client, archive):
             response = client.get_messages('baz', limit=1)
             self.assertEqual(1, len(response['messages']))
             messages = response['messages']
@@ -295,9 +254,16 @@
 
     def get_messages_member_order_test(self, key):
         client = GrackleClient('localhost', 8439)
-        with ForkedFakeService.from_client(client,
-                {'baz': [{'message_id': 'foo', key: '2011-03-25'},
-                 {'message_id': 'bar', key: '2011-03-24'}]}):
+        if key == 'author':
+            header_name = 'from'
+        else:
+            header_name = key
+        archive = {
+            'baz': [
+                make_message('foo', headers={header_name: '2011-03-25'}),
+                make_message('bar', headers={header_name: '2011-03-24'}),
+             ]}
+        with ForkedFakeService.from_client(client, archive):
             response = client.get_messages('baz')
             self.assertIDOrder(['foo', 'bar'], response['messages'])
             response = client.get_messages('baz', order=key)
@@ -313,12 +279,15 @@
         self.get_messages_member_order_test('subject')
 
     def test_get_messages_thread_subject_order(self):
+        archive = {
+            'baz': [
+                make_message('bar', headers={'subject': 'y'}),
+                make_message('qux', headers={'subject': 'z'}),
+                make_message('foo', headers={'subject': 'x',
+                                             'in-reply-to': 'qux'}),
+             ]}
         client = GrackleClient('localhost', 8439)
-        with ForkedFakeService.from_client(client, {'baz': [
-            {'message_id': 'bar', 'subject': 'y'},
-            {'message_id': 'qux', 'subject': 'z'},
-            {'message_id': 'foo', 'subject': 'x', 'in_reply_to': 'qux'},
-            ]}):
+        with ForkedFakeService.from_client(client, archive):
             response = client.get_messages('baz')
             self.assertIDOrder(['bar', 'qux', 'foo'], response['messages'])
             response = client.get_messages('baz', order='subject')
@@ -328,11 +297,14 @@
 
     def test_get_messages_thread_oldest_order(self):
         client = GrackleClient('localhost', 8439)
-        with ForkedFakeService.from_client(client, {'baz': [
-            {'message_id': 'bar', 'date': 'x'},
-            {'message_id': 'qux', 'date': 'z'},
-            {'message_id': 'foo', 'date': 'y', 'in_reply_to': 'qux'},
-            ]}):
+        archive = {
+            'baz': [
+                make_message('bar', headers={'date': 'x'}),
+                make_message('qux', headers={'date': 'z'}),
+                make_message('foo', headers={'date': 'y',
+                                             'in-reply-to': 'qux'}),
+            ]}
+        with ForkedFakeService.from_client(client, archive):
             response = client.get_messages('baz')
             self.assertIDOrder(['bar', 'qux', 'foo'], response['messages'])
             response = client.get_messages('baz', order='date')
@@ -342,12 +314,16 @@
 
     def test_get_messages_thread_newest_order(self):
         client = GrackleClient('localhost', 8439)
-        with ForkedFakeService.from_client(client, {'baz': [
-            {'message_id': 'bar', 'date': 'x'},
-            {'message_id': 'qux', 'date': 'w'},
-            {'message_id': 'foo', 'date': 'y', 'in_reply_to': 'bar'},
-            {'message_id': 'baz', 'date': 'z', 'in_reply_to': 'qux'},
-            ]}):
+        archive = {
+            'baz': [
+                make_message('bar', headers={'date': 'x'}),
+                make_message('qux', headers={'date': 'w'}),
+                make_message('foo', headers={'date': 'y',
+                                             'in-reply-to': 'bar'}),
+                make_message('baz', headers={'date': 'z',
+                                             'in-reply-to': 'qux'}),
+            ]}
+        with ForkedFakeService.from_client(client, archive):
             response = client.get_messages('baz', order='date')
             self.assertIDOrder(
                 ['qux', 'bar', 'foo', 'baz'], response['messages'])
@@ -357,18 +333,19 @@
 
     def test_get_messages_unsupported_order(self):
         client = GrackleClient('localhost', 8439)
-        with ForkedFakeService.from_client(client,
-                {'baz': [{'message_id': 'foo', 'date': '2011-03-25'},
-                 {'message_id': 'bar', 'date': '2011-03-24'}]}):
+        archive = {
+            'baz': [
+                make_message('foo', headers={'date': '2011-03-25'}),
+                make_message('foo', headers={'date': '2011-03-24'}),
+            ]}
+        with ForkedFakeService.from_client(client, archive):
             with ExpectedException(UnsupportedOrder, ''):
                 client.get_messages('baz', order='nonsense')
 
     def test_get_messages_headers_no_headers(self):
         client = GrackleClient('localhost', 8440)
-        with ForkedFakeService.from_client(client,
-            {'baz': [
-                {'message_id': 'foo'}
-            ]}):
+        archive = {'baz': [make_message('foo')]}
+        with ForkedFakeService.from_client(client, archive):
             response = client.get_messages('baz', headers=[
                 'Subject', 'Date', 'X-Launchpad-Message-Rationale'])
         first_message = response['messages'][0]
@@ -377,10 +354,9 @@
 
     def test_get_messages_headers_exclude_headers(self):
         client = GrackleClient('localhost', 8441)
-        with ForkedFakeService.from_client(client,
-            {'baz': [
-                {'message_id': 'foo', 'headers': {'From': 'me'}}
-            ]}):
+        archive = {
+            'baz': [make_message('foo', headers={'From': 'me'})]}
+        with ForkedFakeService.from_client(client, archive):
             response = client.get_messages('baz', headers=[
                 'Subject', 'Date', 'X-Launchpad-Message-Rationale'])
         first_message = response['messages'][0]
@@ -389,10 +365,10 @@
 
     def test_get_messages_headers_include_headers(self):
         client = GrackleClient('localhost', 8442)
-        with ForkedFakeService.from_client(client,
-            {'baz': [
-                {'message_id': 'foo', 'headers': {'From': 'me', 'To': 'you'}}
-            ]}):
+        archive = {
+            'baz': [
+                make_message('foo', headers={'From': 'me', 'To': 'you'})]}
+        with ForkedFakeService.from_client(client, archive):
             response = client.get_messages('baz', headers=[
                 'From', 'To'])
         first_message = response['messages'][0]
@@ -401,21 +377,20 @@
 
     def test_get_messages_max_body_length(self):
         client = GrackleClient('localhost', 8443)
-        with ForkedFakeService.from_client(client,
-            {'baz': [
-                {'message_id': 'foo', 'body': u'abcdefghi'}
-            ]}):
+        archive = {'baz': [make_message('foo', body=u'abcdefghi')]}
+        with ForkedFakeService.from_client(client, archive):
             response = client.get_messages('baz', max_body_length=3)
         first_message = response['messages'][0]
         self.assertEqual('abc', first_message['body'])
 
     def test_include_hidden(self):
         client = GrackleClient('localhost', 8444)
-        with ForkedFakeService.from_client(client,
-            {'baz': [
-                {'message_id': 'foo', 'hidden': True},
-                {'message_id': 'bar', 'hidden': False}
-            ]}):
+        archive = {
+            'baz': [
+                make_message('foo', hidden=True),
+                make_message('bar', hidden=False),
+            ]}
+        with ForkedFakeService.from_client(client, archive):
             response = client.get_messages('baz', include_hidden=True)
             self.assertMessageIDs(['bar', 'foo'], response['messages'])
             response = client.get_messages('baz', include_hidden=False)
@@ -423,9 +398,120 @@
 
     def test_display_type_unknown_value(self):
         client = GrackleClient('localhost', 8445)
-        with ForkedFakeService.from_client(client,
-            {'baz': [
-                {'message_id': 'foo', 'body': u'abcdefghi'}
-            ]}):
+        archive = {'baz': [make_message('foo', body=u'abcdefghi')]}
+        with ForkedFakeService.from_client(client, archive):
             with ExpectedException(UnsupportedDisplayType, ''):
                 client.get_messages('baz', display_type='unknown')
+
+    def test_display_type_headers_only(self):
+        client = GrackleClient('localhost', 8446)
+        archive = {
+            'baz': [
+                make_message('foo', body=u'abcdefghi',
+                             headers={'From': 'me', 'To': 'you'})]}
+        with ForkedFakeService.from_client(client, archive):
+            response = client.get_messages('baz', display_type='headers-only')
+        first_message = response['messages'][0]
+        self.assertEqual('foo', first_message['message_id'])
+        self.assertEqual(
+            archive['baz'][0]['headers'], first_message['headers'])
+        self.assertNotIn('body', first_message)
+
+    def test_display_type_text_only(self):
+        client = GrackleClient('localhost', 8446)
+        archive = {
+            'baz': [
+                make_mime_message(
+                    'foo', 'abcdefghi',
+                    headers={'From': 'me', 'To': 'you'},
+                    attachment_type='text/x-diff')]}
+        with ForkedFakeService.from_client(client, archive):
+            response = client.get_messages('baz', display_type='text-only')
+        first_message = response['messages'][0]
+        self.assertEqual('foo', first_message['message_id'])
+        self.assertEqual('me', first_message['headers']['From'])
+        self.assertEqual('you', first_message['headers']['To'])
+        self.assertEqual(archive['baz'][0]['body'], first_message['body'])
+
+    def test_display_type_all(self):
+        client = GrackleClient('localhost', 8447)
+        archive = {
+            'baz': [
+                make_mime_message(
+                    'foo', 'abcdefghi',
+                    headers={'From': 'me', 'To': 'you'},
+                    attachment_type='text/x-diff')]}
+        with ForkedFakeService.from_client(client, archive):
+            response = client.get_messages('baz', display_type='all')
+        first_message = response['messages'][0]
+        self.assertEqual('foo', first_message['message_id'])
+        self.assertEqual('me', first_message['headers']['From'])
+        self.assertEqual('you', first_message['headers']['To'])
+        self.assertEqual(archive['baz'][0]['body'], first_message['body'])
+
+    def test_date_range(self):
+        client = GrackleClient('localhost', 8448)
+        archive = {
+            'baz': [
+                make_mime_message(
+                    'foo', 'abcdefghi', headers={'date': '2011-12-31'}),
+                make_mime_message(
+                    'bar', 'abcdefghi', headers={'date': '2012-01-01'}),
+                make_mime_message(
+                    'qux', 'abcdefghi', headers={'date': '2012-01-15'}),
+                make_mime_message(
+                    'naf', 'abcdefghi', headers={'date': '2012-01-31'}),
+                make_mime_message(
+                    'doh', 'abcdefghi', headers={'date': '2012-02-01'}),
+                    ]}
+        with ForkedFakeService.from_client(client, archive):
+            response = client.get_messages(
+                'baz', date_range='2012-01-01..2012-01-31')
+        ids = sorted(m['message_id'] for m in response['messages'])
+        self.assertEqual(['bar', 'naf', 'qux'], ids)
+
+    def test_date_range_unparsabledaterange(self):
+        client = GrackleClient('localhost', 8449)
+        archive = {'baz': [make_message('foo', body=u'abcdefghi')]}
+        with ForkedFakeService.from_client(client, archive):
+            with ExpectedException(UnparsableDateRange, ''):
+                client.get_messages('baz', date_range='2012-01-01')
+
+    def test_date_range_unparsabledaterange_missing_part(self):
+        client = GrackleClient('localhost', 8450)
+        archive = {'baz': [make_message('foo', body=u'abcdefghi')]}
+        with ForkedFakeService.from_client(client, archive):
+            with ExpectedException(UnparsableDateRange, ''):
+                client.get_messages('baz', date_range='2012-01-01..')
+
+    def test_date_range_unparsabledaterange_extra_part(self):
+        client = GrackleClient('localhost', 8451)
+        archive = {'baz': [make_message('foo', body=u'abcdefghi')]}
+        with ForkedFakeService.from_client(client, archive):
+            with ExpectedException(UnparsableDateRange, ''):
+                client.get_messages('baz', date_range='2012-01..12-02..12-03')
+
+
+class TestHideMessages(TestCase):
+
+    def test_hide_message_true(self):
+        client = GrackleClient('localhost', 8470)
+        archive = {
+            'baz': [
+                make_message('foo', hidden=False),
+            ]}
+        with ForkedFakeService.from_client(client, archive):
+            response = client.hide_message('baz', 'foo', hidden=True)
+        self.assertEqual('foo', response['message_id'])
+        self.assertIs(True, response['hidden'])
+
+    def test_hide_message_false(self):
+        client = GrackleClient('localhost', 8470)
+        archive = {
+            'baz': [
+                make_message('foo', hidden=True),
+            ]}
+        with ForkedFakeService.from_client(client, archive):
+            response = client.hide_message('baz', 'foo', hidden=False)
+        self.assertEqual('foo', response['message_id'])
+        self.assertIs(False, response['hidden'])

